/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file dist_rose_tree.h
 * @brief The header file of class @a dist_rose_tree; a parallel implementation
 *        of rose trees via arrangement to binary trees.
 * @author Kiminori MATSUZAKI
 */

#ifndef __DIST_ROSE_TREE_H__
#define __DIST_ROSE_TREE_H__

#include "dist_tree.h"
#include "rose_value.h"

/**
 * @brief A parallel implementation of rose trees via arrangement to binary
 *        trees.
 */
template< typename A >
class dist_rose_tree
{
  friend class rose_tree_skeletons;

  ////////////////////////////////////////////////////////////////////////
  // Member variables
  
  /** @brief Tree arranged as a binary tree. */
private:
  dist_tree< rose_value< A > > *btree;

  ////////////////////////////////////////////////////////////////////////
  // Constructor and Destructor

  /**
   * @brief Construct by specifying the structure arranged in a binary tree.
   *
   * For performance reasons, this constructor does not create any clone
   * for the input binary tree, and the binary tree will be freed by
   * the destructor of this class. 
   */
private:
  dist_rose_tree( dist_tree< rose_value< A > > *btree );

  /**
   * @brief Destructor.
   */
public:
  ~dist_rose_tree( );

  ////////////////////////////////////////////////////////////////////////
  // Input and Output
  
  /**
   * @brief Construct by reading from file.
   * @param filename : The file to read from. 
   */
public:
  static dist_rose_tree< A > *read_from_file( const char *filename );

  /**
   * @brief Output the tree into a data file
   *
   * @param filename: Output data file
   * @retval true : The tree is correctly written to the file.
   * @retval false : Something wrong occurred.
   */
public:
  bool write_to_file( const char *filename ) const;
  
  /**
   * @brief Simple printing function
   */
public:
  void print( ) const;

};

#include "dist_rose_tree.tpp"

#endif // __DIST_ROSE_TREE_H__
