/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file rose_value.h
 * @brief The header file of class @a rose_value; the value of a node on
 *        binary tree arranged from a rose tree.
 * @author Kiminori MATSUZAKI
 */

#ifndef __ROSE_VALUE_H__
#define __ROSE_VALUE_H__

#include <iostream>

/**
 * @brief A template class for the value of a node of a binary tree
 *        arranged from a rose tree as mentioned in CMPP04 paper.
 */
template< typename A >
class rose_value {
public:
  ////////////////////////////////////////////////////////////////////////
  // constant
  /** @brief Enumulating type added to the node value.
   *         See our CMPP04 paper for the meaning of each keywords.
   */
  typedef enum {
    ORGLEAF = 0,
    ORGNODE = 1,
    ORGNIL = 2,
    DUMMY = 3,
  } valuetype_t;
  
  ////////////////////////////////////////////////////////////////////////
  // member variables
public:
  /** @brief The type of this value. */
  valuetype_t valuetype;

  /** @brief The real value */
  A value;

  ////////////////////////////////////////////////////////////////////////
  // constructors and destructor
  /**
   * @brief Construct by specifying @c value and @c valuetype.
   */
  rose_value( const A& value, valuetype_t valuetype );

  /**
   * @brief Default-constructor for containers.
   */
  rose_value( );

  /**
   * @brief Destructor
   */
  ~rose_value( );
};

template< typename A >
inline std::ostream& operator<<( std::ostream& s, const rose_value< A >& a );

template< typename A >
inline std::istream& operator>>( std::istream& s, rose_value< A >& a );

#include "rose_value.tpp"

#endif /* __ROSE_VALUE_H__ */
