/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file test_dist_rose_tree.cpp
 * @brief Test code for dist_rose_tree.h and dist_rose_tree.tpp
 * @author Kiminori MATSUZAKI
 */

#include <assert.h>
#include <iostream>

#include "dist_rose_tree.h"

#define DEBUG_OUT 0

#if DEBUG_OUT
#define debug( x )   x
#else
#define debug( x )
#endif /* DEBUG_OUT */

void test_dist_rose_tree_read_from_file_and_print( )
{
  const char filename[] = "dist_rose_tree_data.txt";
  dist_rose_tree< double > *rtree
    = dist_rose_tree< double >::read_from_file( filename );
  assert( rtree );

  debug( rtree->print( ) );

  if ( rtree ) delete rtree;
}

void test_dist_rose_tree_read_from_file_and_write_to_file( )
{
  const char filename_in[]  = "dist_rose_tree_data.txt";
  const char filename_out[] = "result_test_dist_rose_tree_read_and_write.txt";

  dist_rose_tree< double > *rtree
    = dist_rose_tree< double >::read_from_file( filename_in );
  assert( rtree );
  
  assert( rtree->write_to_file( filename_out ) );
  if ( skeleton::rank == 0 ) {
    std::cerr << "Check the output of write_to_file: " << filename_out
	      << std::endl;
  }

  if ( rtree ) delete rtree;
}

int SketoMain( int, char ** )
{
  if ( skeleton::rank == 0 ) {
    cout << "Testing " __FILE__ << endl;
  }
  
  test_dist_rose_tree_read_from_file_and_print( );
  test_dist_rose_tree_read_from_file_and_write_to_file( );

  if ( skeleton::rank == 0 ) {
    cout << "   ---> OK " __FILE__ << endl;
  }
  return 0;
}
