/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file test_node.cpp
 * @brief Test code for node.h and node.tpp
 * @author Kiminori MATSUZAKI
 */

#include <assert.h>
#include <iostream>

#include <dist_tree.h>
#include <node.h>

using namespace std;

void test_node_constructor_and_destructor( )
{
  {
    node< int > *intNode = new node< int >( 100 );
    
    assert( 100 == intNode->value );
    assert( node< int >::NORMAL == intNode->nodetype );
    
    delete intNode;
  }

  {
    node< double > *doubleNode
      = new node< double >( 20.0, node< double >::CUTNODE );

    assert( 20.0 == doubleNode->value );
    assert( node< double >::CUTNODE == doubleNode->nodetype );

    delete doubleNode;
  }
}

void test_node_isLeaf( )
{
  node< int > *node1 = new node< int >( 0 );
  node< int > *node2 = new node< int >( 0 );
  node< int > *node3 = new node< int >( 0 );

  node1->left = node2;
  node1->right = node3;

  assert( false == node1->isLeaf( ) );
  assert( true  == node2->isLeaf( ) );
  assert( true  == node3->isLeaf( ) );

  delete node3;
  delete node2;
  delete node1;
}

int SketoMain( int, char** )
{
  if ( skeleton::rank == 0 ) {
    cout << "Testing " __FILE__ << endl;

    test_node_constructor_and_destructor( );
    test_node_isLeaf( );

    cout << "   ---> OK " __FILE__ << endl;
  }

  return 0;
}
