/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file test_rose_reduce_matrix.cpp
 * @brief Test code for rodece skeleton on rose trees. 
 * @author Kiminori MATSUZAKI
 */

#include <iostream>
#include "matrix2.h"

class Func_Reduce_Normal_k1 {
public:
  Matrix2< int > operator()( const int &a ) const
  {
    return Matrix2< int >( a / 2 + 1, a / 3 + 0, a / 4 + 0, a / 5 + 2 );
  }
};

class Func_Reduce_Normal_k2 {
public:
  int operator()( const int& b ) const { return (15 - b) / 5 + 1; }
};

class Func_Reduce_Normal_oplus {
public:
  Matrix2< int > operator()( const int &lhs,
			     const Matrix2< int > &rhs ) const {
    return rhs * lhs;
  }
};

class Func_Reduce_Normal_otimes {
public:
  Matrix2< int > operator()( const Matrix2< int > &lhs,
			     const Matrix2< int > &rhs ) const {
    return lhs * rhs;
  }
};

int main( void )
{
  Func_Reduce_Normal_k1     k1;
  Func_Reduce_Normal_k2     k2;
  Func_Reduce_Normal_oplus  oplus;
  Func_Reduce_Normal_otimes otimes;
  
  // [4] := e
  std::cout << "result is [4] " << Matrix2< int >( 1, 0, 0, 1 ) << std::endl;

  // [3] := k2 4 + ( ( k2 9 + k1 13) * k1 10 )
  Matrix2< int > v3
    = oplus( k2( 4 ), otimes( oplus( k2( 9 ), k1( 13 ) ), k1( 10 ) ) );
  std::cout << "result is [3] " << v3 << std::endl;

  // [2] := ( k2 3 + ( k1 7 * k1 8 ) ) * v3
  Matrix2< int > v2
    = otimes( oplus( k2( 3 ), otimes( k1( 7 ), k1( 8 ) ) ), v3 );
  std::cout << "result is [2] " << v2 << std::endl;

  // [1] := k2 2 + ( k1 5 * ( k2 6 + ( k1 11 * k1 12 ) ) )
  Matrix2< int > v1
    = oplus( k2( 2 ), otimes( k1( 5 ), oplus( k2( 6 ), otimes( k1( 11), k1( 12 ) ) ) ) );
  std::cout << "result is [1] " << v1 << std::endl;

  // [0] := k2 1 + ( v1 * v2 )
  Matrix2< int > v0
    = oplus( k2( 1 ), otimes( v1, v2 ) );
  std::cout << "result is [0] " << v0 << std::endl;

  return 0;
}

  
