/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file compose.h
 * @brief The header for compose functions.
 * @author Yoshiki AKASHI (yoshiki@ipl.cs.uec.ac.jp)
 */

#ifndef __COMPOSE_H__
#define __COMPOSE_H__

#include "functions.h"

template< typename F, typename G >
class composite_unary_function
  : public unary_function< typename G::argument_type,
			   typename F::result_type >
{
  F f;
  G g;
public:
  composite_unary_function( const F &f_, const G &g_ ) : f( f_ ), g( g_ )
  { }
  typename F::result_type operator()( typename G::argument_type x ) const
  {
    return f( g( x ) );
  }
};

template< typename F, typename G >
class composite_unary_ptr_function
  : public unary_ptr_function< typename F::first_argument_type,
			       typename G::second_argument_type >
{
  F f;
  G g;
public:
  composite_unary_ptr_function( const F &f_, const G &g_ ) : f( f_ ), g( g_ )
  { }
  void operator()( typename F::first_argument_type *r,
		   const typename G::second_argument_type *x ) const
  {
    typename G::first_argument_type a;
    g( &a, x );
    f( r, &a );
  }
};

template< typename F, typename G >
class composite_binary_function
  : public binary_function< typename G::first_argument_type,
			    typename G::second_argument_type,
			    typename F::result_type >
{
  F f;
  G g;
public:
  composite_binary_function( const F &f_, const G &g_ ) : f( f_ ), g( g_ )
  {}
  typename F::result_type operator()( typename G::first_argument_type x,
				      typename G::second_argument_type y ) const
  {
    return f( g( x, y ) );
  }
};

template< typename F, typename G >
inline composite_unary_function< F, G >
compose_unary_functions( const F &f, const G &g )
{
  return composite_unary_function< F, G >( f, g );
}

template< typename F, typename G >
inline composite_unary_ptr_function< F, G >
compose_unary_ptr_functions( const F &f, const G &g )
{
  return composite_unary_ptr_function< F, G >( f, g );
}

template< typename F, typename G >
inline composite_binary_function< F, G >
compose_binary_functions( const F &f, const G &g )
{
  return composite_binary_function< F, G >( f, g );
}

#endif // __COMPOSE_H__
