/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file line_of_sight.cpp
 * @brief This program search lines which you can see.
 * @author Yoshiki AKASHI (yoshiki@ipl.cs.uec.ac.jp)
 */

#include <cstdlib>
#include <cfloat>
#include <ctime>
#include "list_skeletons.h"

const int SIZE = 100;

struct Gen : public unary_function<int, double> {
  double operator()(int index) const {
    return static_cast< double >((rand() % 10) * (index + 1));
  }
} gen;

struct Div : public binary_function<int, double, double> {
  double operator()(int index, double x) const {
    return x / (static_cast<double>(index) + 1.0);
  }
} divide;

struct Max : public binary_function<double, double, double> {
   double operator()(double x, double y) const {
     return (x > y) ? x : y;
   }
} max_func;
const double unit_of_max = -DBL_MAX;

struct Eqfilter : public binary_function<double, double, int> {
  int operator()(double x, double y) const {
    return (x == y) ? 1 : 0;
  }
} eqfilter;

int SketoMain(int, char **)
{
  // generate data
  srand(time(NULL));
  dist_list<double> *as = new dist_list<double>(gen, SIZE);

  // calculate angle
  list_skeletons::mapindex_ow(divide, as);

  // calculate max angle before oneself
  dist_list< double > *bs = list_skeletons::scan(max_func, unit_of_max, as);

  // we drop the first element in the bs.
  dist_list<double> *cs;
  {
    double array[SIZE+1];
    list_skeletons::gather(bs, array);
    cs = new dist_list<double>(SIZE, array+1);
  }
  delete bs;

  // if the angle equal max angle, you can see it.
  dist_list<int> *result = list_skeletons::zipwith(eqfilter, as, cs);
  delete as;
  delete cs;

  list_skeletons::print(result);
  delete result;
  
  return 0;
}
