/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file variance.cpp
 * @brief This program compute average and variance.
 * @author Yoshiki AKASHI (yoshiki@ipl.cs.uec.ac.jp)
 */

#include <iostream>
#include "list_skeletons.h"

const int SIZE = 1000;

struct Gen : public unary_function<int, double> {
  double operator()(int index) const { return static_cast<double>(index); }
} gen;

struct Add : public binary_function<double, double, double> {
  double operator()(double x, double y) const { return x + y; }
} add;
const double add_unit = 0.0;

struct Sqr : public unary_function<double, double> {
  double operator()(double x) const { return x * x; }
} sqr;

struct Sub : public unary_function<double, double> {
  double val;
  Sub(double val_) : val(val_){ }
  double operator()(double x) const { return x - val; }
};

int SketoMain(int argc, char **argv)
{
  // generate data
  dist_list<double> *as = new dist_list<double>(gen, SIZE);

  // calculate average
  double ave = list_skeletons::reduce(add, add_unit, as) / SIZE;
  
  // calculate variance
  Sub sub(ave);
  list_skeletons::map_ow(sub, as);
  list_skeletons::map_ow(sqr, as);
  double var = list_skeletons::reduce(add, add_unit, as) / SIZE;

  // output results
  if(skeleton::rank == 0){
    std::cout << "average:" << ave << "\n" << "variance:" << var << "\n";
  }

  return 0;
}
