/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/***
 * @file fnorm.cpp
 * @brief The Frobenius Norm.  
 * @author Kento EMOTO
 */
#include <cstdlib>
#include <iostream>
#include <cmath>
#include "matrix_skeletons.h"
#include "primitive_functions.h"
using namespace primitive_functions;

double fnorm(const matrix<double> &mat){
	dist_matrix<double> dmat(&mat);
	dist_matrix<double> *dmat2 = matrix_skeletons::map(Square<double>(), &dmat);
	double *ss = matrix_skeletons::reduce(Add<double>(), Add<double>(), dmat2);
	double ret = sqrt(*ss);
	delete ss;
	delete dmat2;
	return ret;
}


int SketoMain( int argc, char **argv )
{
	int n = 1000;
	if(argc > 1){
		n = atoi(argv[1]);
	}
	matrix<double> mat(n, n);
	mat.generate(GenSub<double>());
	double norm = fnorm(mat);
	if(skeleton::rank==0){
		std::cout << "#procs = "<< skeleton::procs << std::endl;
		std::cout << "matrix size = " << n << "x" << n << std::endl;
		std::cout << "The Frobenius Norm = " << norm << std::endl;
		std::cout << "the matrix = " << std::endl;
		std::cout<< mat<<std::endl;
	}
	return 0;
}
