/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/***
 * @file mulssp.cpp
 * @brief Maximum Upper-Left Submatrix Sum Problem.  by Kento EMOTO
 * @author Kento EMOTO
 */
#include <iostream>
#include <cstdlib>
#include "matrix_skeletons.h"
#include "primitive_functions.h"
using namespace primitive_functions;

double mulssp(const matrix<double> &mat){
	dist_matrix<double> dmat(&mat);
	dist_matrix<double> *dmat2 = matrix_skeletons::scan(Add<double>(), Add<double>(), &dmat);
	double *ss = matrix_skeletons::reduce(Max<double>(), Max<double>(), dmat2);
	double ret = *ss;
	delete ss;
	delete dmat2;
	return ret;
}

int SketoMain( int argc, char **argv )
{

	int n = 1000;
	if(argc > 1){
		n = atoi(argv[1]);
	}
	matrix<double> mat(n, n);
	mat.generate(GenSub<double>());
	double mulss = mulssp(mat);
	if(skeleton::rank==0){
		std::cout << "#procs = "<< skeleton::procs << std::endl;
		std::cout << "matrix size = " << n << "x" << n << std::endl;
		std::cout << "Maximum Upper-Left Submatrix Sum = " << mulss << std::endl;
		std::cout << "the matrix = " << std::endl;
		std::cout<< mat<<std::endl;
	}
	
	return 0;
}

 
