/***
 * Copyright (c) 2005, SkeTo Project
 * All rights reserved.
 */
/**
 * @file create_int_tree.cpp
 * @brief Create a tree of int and distribute it.
 * @author Kiminori MATSUZAKI
 */

#include <iostream>
#include <cstdio>
#include <cstdlib>
#include <vector>

#include <node.h>
#include <tree_util.h>

using namespace std;

const int numNodes = 1000001;
// const int numNodes = 41;
// const int seed     = 2;

const int procs[] = { 1, 81, 0 };
const int seeds[] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 0 };
// const int seeds[] = { 1, 0 };

int main( void ) {
  for ( int seedindex = 0; seeds[ seedindex ] != 0; seedindex++ ) {
    int seed = seeds[ seedindex ];
    srand( seed );

    std::vector< node< int >* > list;
    node< int > *root = new node< int >( rand( ) % 10 );
    list.push_back( root );
    
    for ( int i = 1; i < numNodes; i+= 2 ) {
      int p = rand( ) % list.size( );
      node< int > *cur = list[ p ];
      
      cur->left  = new node< int >( rand( ) % 10 );
      cur->right = new node< int >( rand( ) % 10 );
      //    cur->left  = new node< int >( rand( ) % 10 );
      //    cur->right = new node< int >( rand( ) % 10 );
      
      list[ p ] = cur->left;
      list.push_back( cur->right );
    }
    
    cerr << "data creation end. then distribute" << endl;
    
    char buf[ 1024 ];
    //  for ( int i = 1; i < 100; i+= (i / 20) + 3 ) {
    // for ( int i = 1; i < 5; i++ ) {
    for ( int i = 0; procs[ i ] != 0; i++ ) {
      printf( "%d\n"	   , procs[ i ] );
      
      sprintf( buf, "treedata.%d.%02d.%02d", numNodes, seed, procs[ i ] );
      assert( tree_util::dist_and_write_to_file( root, procs[ i ], buf ) );
      // sprintf( buf, "treedata.%d.%d.%02d", numNodes, seed, i );
      // assert( tree_util::dist_and_write_to_file( root, i, buf ) );
    }

    tree_util::free_all_tree( root );
  }

  return 0;
}

#if 0
int main( void ) {
  srand( seed );
  
  node< int > *root = new node< int >( rand( ) % 10 );
  for ( int i = 1; i < numNodes; i+= 2 ) {
    node< int > *cur = root;
    while( !cur->isLeaf( ) ) {
      if ( rand( ) % 2 ) {
	cur = cur->left;
      } else {
	cur = cur->right;
      }
    }
    cur->left  = new node< int >( rand( ) % 10 );
    cur->right = new node< int >( rand( ) % 10 );
  }

  cerr << "data creation end. then distribute" << endl;
  
  char buf[ 1024 ];
  for ( int i = 1; i < 50; i+=2 ) {
    sprintf( buf, "treedata.%d.%d.%02d", numNodes, seed, i );
    assert( tree_util::dist_and_write_to_file( root, i, buf ) );
  }
}
#endif 
	
  
